@@ -1,4 +1,4 @@
-/* Copyright 1995-2004,2006-2015,2017-2018
+/* Copyright 1995-2004,2006-2015,2017-2019
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -626,9 +626,9 @@ static scm_t_off
 fport_seek (SCM port, scm_t_off offset, int whence)
 {
   scm_t_fport *fp = SCM_FSTREAM (port);
-  off_t_or_off64_t result;
+  scm_t_off result;
 
-  result = lseek_or_lseek64 (fp->fdes, offset, whence);
+  result = lseek (fp->fdes, offset, whence);
 
   if (result == -1)
     scm_syserror ("fport_seek");
