@@ -3242,11 +3242,93 @@ VM_NAME (scm_thread *thread)
       NEXT (1);
     }
 
-  VM_DEFINE_OP (155, unused_155, NULL, NOP)
-  VM_DEFINE_OP (156, unused_156, NULL, NOP)
-  VM_DEFINE_OP (157, unused_157, NULL, NOP)
-  VM_DEFINE_OP (158, unused_158, NULL, NOP)
-  VM_DEFINE_OP (159, unused_159, NULL, NOP)
+  /* call-f64<-f64 dst:12 src:12 IDX:32
+   *
+   * Call the double-returning instrinsic with index IDX, passing SCM
+   * local SRC as argument.  Place the double result in DST.
+   */
+  VM_DEFINE_OP (155, call_f64_from_f64, "call-f64<-f64", DOP2 (X8_S12_S12, C32))
+    {
+      uint16_t dst, src;
+      scm_t_f64_from_f64_intrinsic intrinsic;
+
+      UNPACK_12_12 (op, dst, src);
+      intrinsic = intrinsics[ip[1]];
+
+      /* We assume these instructions can't throw an exception.  */
+      SP_SET_F64 (dst, intrinsic (SP_REF_F64 (src)));
+
+      NEXT (2);
+    }
+
+  /* call-f64<-f64-f64 dst:8 a:8 b:8 IDX:32
+   *
+   * Call the double-returning instrinsic with index IDX, passing SCM
+   * locals A and B as arguments.  Place the double result in DST.
+   */
+  VM_DEFINE_OP (156, call_f64_from_f64_f64, "call-f64<-f64-f64", DOP2 (X8_S8_S8_S8, C32))
+    {
+      uint8_t dst, a, b;
+      scm_t_f64_from_f64_f64_intrinsic intrinsic;
+
+      UNPACK_8_8_8 (op, dst, a, b);
+      intrinsic = intrinsics[ip[1]];
+
+      /* We assume these instructions can't throw an exception.  */
+      SP_SET_F64 (dst, intrinsic (SP_REF_F64 (a), SP_REF_F64 (b)));
+
+      NEXT (2);
+    }
+
+  /* allocate-pointerless-words dst:12 count:12
+   *
+   * Allocate a fresh object consisting of COUNT words and store it into
+   * DST.  The result will not be traced by GC.  COUNT is a u64 local.
+   */
+  VM_DEFINE_OP (157, allocate_pointerless_words, "allocate-pointerless-words", DOP1 (X8_S12_S12))
+    {
+      uint16_t dst, size;
+
+      UNPACK_12_12 (op, dst, size);
+
+      SYNC_IP ();
+      SP_SET (dst, CALL_INTRINSIC (allocate_pointerless_words,
+                                   (thread, SP_REF_U64 (size))));
+      NEXT (1);
+    }
+
+  /* allocate-words/immediate dst:12 count:12
+   *
+   * Allocate a fresh object consisting of COUNT words and store it into
+   * DST.  The result will not be traced by GC.  COUNT is an immediate.
+   */
+  VM_DEFINE_OP (158, allocate_pointerless_words_immediate, "allocate-pointerless-words/immediate", DOP1 (X8_S12_C12))
+    {
+      uint16_t dst, size;
+
+      UNPACK_12_12 (op, dst, size);
+
+      SYNC_IP ();
+      SP_SET (dst, CALL_INTRINSIC (allocate_pointerless_words, (thread, size)));
+
+      NEXT (1);
+    }
+
+  /* s64->f64 dst:12 src:12
+   *
+   * Convert an s64 value to a double-precision floating-point value.
+   */
+  VM_DEFINE_OP (159, s64_to_f64, "s64->f64", DOP1 (X8_S12_S12))
+    {
+      uint16_t dst, src;
+
+      UNPACK_12_12 (op, dst, src);
+
+      SP_SET_F64 (dst, (double) SP_REF_S64 (src));
+
+      NEXT (1);
+    }
+
   VM_DEFINE_OP (160, unused_160, NULL, NOP)
   VM_DEFINE_OP (161, unused_161, NULL, NOP)
   VM_DEFINE_OP (162, unused_162, NULL, NOP)
