@@ -47,10 +47,13 @@
       (set! x (maybe-verify (pass x)))))
   (define (resolve* x) (resolve-primitives x env))
   (define (peval* x) (peval x env))
+  (define (letrectify* x)
+    (let ((seal? (kw-arg-ref opts #:seal-private-bindings? #f)))
+      (letrectify x #:seal-private-bindings? seal?)))
   (maybe-verify x)
   (run-pass resolve*           #:resolve-primitives? #t)
   (run-pass expand-primitives  #:expand-primitives?  #t)
-  (run-pass letrectify         #:letrectify?         #t)
+  (run-pass letrectify*        #:letrectify?         #t)
   (set! x (fix-letrec x))
   (run-pass peval*             #:partial-eval?       #t)
   x)
@@ -67,4 +70,5 @@
   '((#:resolve-primitives? 2)
     (#:expand-primitives? 1)
     (#:letrectify? 2)
+    (#:seal-private-bindings? 3)
     (#:partial-eval? 1)))
