@@ -1,7 +1,7 @@
 ;;;; (statprof) -- a statistical profiler for Guile
 ;;;; -*-scheme-*-
 ;;;;
-;;;; 	Copyright (C) 2009, 2010, 2011, 2013-2017  Free Software Foundation, Inc.
+;;;; 	Copyright (C) 2009, 2010, 2011, 2013-2018  Free Software Foundation, Inc.
 ;;;;    Copyright (C) 2004, 2009 Andy Wingo <wingo at pobox dot com>
 ;;;;    Copyright (C) 2001 Rob Browning <rlb at defaultvalue dot org>
 ;;;; 
@@ -65,7 +65,6 @@
 
             statprof-display
             statprof-display-anomalies
-            statprof-display-anomolies ; Deprecated spelling.
 
             statprof-fetch-stacks
             statprof-fetch-call-tree
@@ -92,26 +91,16 @@
 ;;; distinguish between different closures which share the same code,
 ;;; but that is usually what we want anyway.
 ;;;
-;;; One case in which we do want to distinguish closures is the case of
-;;; primitive procedures.  If slot 0 in the frame is a primitive
-;;; procedure, we record the procedure's name into the buffer instead of
-;;; the IP.  It's fairly cheap to check whether a value is a primitive
-;;; procedure, and then get its name, as its name is stored in the
-;;; closure data.  Calling procedure-name in the stack sampler isn't
-;;; something you want to do for other kinds of procedures, though, as
-;;; that involves grovelling the debug information.
-;;;
 ;;; The other part of data collection is the exact call counter, which
 ;;; uses the VM's "apply" hook to record each procedure call.
 ;;; Naturally, this is quite expensive, and it is off by default.
 ;;; Running code at every procedure call effectively penalizes procedure
 ;;; calls.  Still, it's useful sometimes.  If the profiler state has a
 ;;; call-counts table, then calls will be counted.  As with the stack
-;;; counter, usually the key in the hash table is the code pointer of
-;;; the procedure being called, except for primitive procedures, in
-;;; which case it is the name of the primitive.  The call counter can
-;;; also see calls of non-programs, for example in the case of
-;;; applicable structs.  In that case the key is the procedure itself.
+;;; counter, the key in the hash table is the code pointer of the
+;;; procedure being called.  The call counter can also see calls of
+;;; non-programs, for example in the case of applicable structs.  In
+;;; that case the key is the procedure itself.
 ;;;
 ;;; After collection is finished, the data can be analyzed.  The first
 ;;; step is usually to run over the stack traces, tabulating sample
@@ -250,8 +239,7 @@
       (set-buffer! state buffer)
       (set-buffer-pos! state (1+ pos)))
      (else
-      (write-sample-and-continue
-       (frame-instruction-pointer-or-primitive-procedure-name frame))))))
+      (write-sample-and-continue (frame-instruction-pointer frame))))))
 
 (define (reset-sigprof-timer usecs)
   ;; Guile's setitimer binding is terrible.
@@ -297,7 +285,7 @@
 (define (count-call frame)
   (let ((state (existing-profiler-state)))
     (unless (inside-profiler? state)
-      (let* ((key (frame-instruction-pointer-or-primitive-procedure-name frame))
+      (let* ((key (frame-instruction-pointer frame))
              (handle (hashv-create-handle! (call-counts state) key 0)))
         (set-cdr! handle (1+ (cdr handle)))))))
 
@@ -325,7 +313,7 @@ than @code{statprof-stop}, @code{#f} otherwise."
         (set-prev-sigprof-handler! state (car prev)))
       (reset-sigprof-timer (if (zero? rpt) (sampling-period state) rpt))
       (when (call-counts state)
-        (add-hook! (vm-apply-hook) count-call)
+        (vm-add-apply-hook! count-call)
         (set-vm-trace-level! (1+ (vm-trace-level))))
       #t)))
   
@@ -338,7 +326,7 @@ than @code{statprof-stop}, @code{#f} otherwise."
   (when (zero? (profile-level state))
     (when (call-counts state)
       (set-vm-trace-level! (1- (vm-trace-level)))
-      (remove-hook! (vm-apply-hook) count-call))
+      (vm-remove-apply-hook! count-call))
     (set-gc-time-taken! state
                         (- (assq-ref (gc-stats) 'gc-time-taken)
                            (gc-time-taken state)))
@@ -448,42 +436,26 @@ always collects full stacks.)"
               (hashv-set! table entry data)
               data))))
 
-    (define (callee->call-data callee)
-      (cond
-       ((number? callee) (addr->call-data callee))
-       ((hashv-ref table callee))
-       (else
-        (let ((data (make-call-data
-                     (cond ((procedure? callee) (procedure-name callee))
-                           ;; a primitive
-                           ((symbol? callee) callee)
-                           (else #f))
-                     (with-output-to-string (lambda () (write callee)))
-                     #f
-                     (and call-counts (hashv-ref call-counts callee))
-                     0
-                     0)))
-          (hashv-set! table callee data)
-          data))))
-
     (when call-counts
       (hash-for-each (lambda (callee count)
-                       (callee->call-data callee))
+                       (unless (number? callee)
+                         (error "unexpected callee" callee))
+                       (addr->call-data callee))
                      call-counts))
 
     (let visit-stacks ((pos 0))
       (cond
        ((< pos len)
         (let ((pos (if call-counts
-                       (skip-count-call buffer pos len)
-                       pos)))
+                        (skip-count-call buffer pos len)
+                        pos)))
           (inc-call-data-self-sample-count!
-           (callee->call-data (vector-ref buffer pos)))
+           (addr->call-data (vector-ref buffer pos)))
           (let visit-stack ((pos pos))
             (cond
              ((vector-ref buffer pos)
-              => (lambda (callee)
-                   (inc-call-data-cum-sample-count! (callee->call-data callee))
+              => (lambda (ip)
+                   (inc-call-data-cum-sample-count! (addr->call-data ip))
                    (visit-stack (1+ pos))))
              (else
               (visit-stacks (1+ pos)))))))
@@ -531,11 +503,7 @@ none is available."
     (error "Can't call statprof-proc-call-data while profiler is running."))
   (unless (program? proc)
     (error "statprof-call-data only works for VM programs"))
-  (let* ((code (program-code proc))
-         (key (if (primitive-code? code)
-                  (procedure-name proc)
-                  code)))
-    (hashv-ref (stack-samples->procedure-data state) key)))
+  (hashv-ref (stack-samples->procedure-data state) (program-code proc)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Stats
@@ -652,8 +620,8 @@ optional @var{port} argument is passed, uses the current output port."
       (for-each display-stats-line sorted-stats)
 
       (display "---\n" port)
-      (format #t "Sample count: ~A\n" (statprof-sample-count state))
-      (format #t "Total time: ~A seconds (~A seconds in GC)\n"
+      (format port "Sample count: ~A\n" (statprof-sample-count state))
+      (format port "Total time: ~A seconds (~A seconds in GC)\n"
               (statprof-accumulated-time state)
               (/ (gc-time-taken state)
                  1.0 internal-time-units-per-second))))))
@@ -677,11 +645,6 @@ statistics.@code{}"
   (format #t "Total time: ~A\n" (statprof-accumulated-time state))
   (format #t "Sample count: ~A\n" (statprof-sample-count state)))
 
-(define (statprof-display-anomolies)
-  (issue-deprecation-warning "statprof-display-anomolies is a misspelling. "
-                             "Use statprof-display-anomalies instead.")
-  (statprof-display-anomalies))
-
 (define* (statprof-accumulated-time #:optional (state
                                                 (existing-profiler-state)))
   "Returns the time accumulated during the last statprof run.@code{}"
@@ -895,49 +858,6 @@ operation is somewhat expensive."
           (statprof-stop state)
           (statprof-display port state #:style display-style))))))
 
-(begin-deprecated
- (define-macro (with-statprof . args)
-   "Profile the expressions in the body, and return the body's return values.
-
-Keyword arguments:
-
-@table @code
-@item #:display-style
-Set the display style, either @code{'flat} or @code{'tree}.
-
-@item #:loop
-Execute the body @var{loop} number of times, or @code{#f} for no looping
-
-default: @code{#f}
-@item #:hz
-Sampling rate
-
-default: @code{20}
-@item #:count-calls?
-Whether to instrument each function call (expensive)
-
-default: @code{#f}
-@end table"
-   (define (kw-arg-ref kw args def)
-     (cond
-      ((null? args) (error "Invalid macro body"))
-      ((keyword? (car args))
-       (if (eq? (car args) kw)
-           (cadr args)
-           (kw-arg-ref kw (cddr args) def)))
-      ((eq? kw #f def) ;; asking for the body
-       args)
-      (else def))) ;; kw not found
-   (issue-deprecation-warning
-    "`with-statprof' is deprecated.  Use `statprof' instead.")
-   `((@ (statprof) statprof)
-     (lambda () ,@(kw-arg-ref #f args #f))
-     #:display-style ,(kw-arg-ref #:display-style args ''flat)
-     #:loop ,(kw-arg-ref #:loop args 1)
-     #:hz ,(kw-arg-ref #:hz args 100)
-     #:count-calls? ,(kw-arg-ref #:count-calls? args #f)))
- (export with-statprof))
-
 (define* (gcprof thunk #:key (loop 1) full-stacks? (port (current-output-port)))
   "Do an allocation profile of the execution of @var{thunk}.
 
